#include <glpk.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <vector>

void transformCppMatrixToGLPKMatrix(std::vector<std::vector<int>> matrix, std::vector<int> & rows, std::vector<int> &columns, std::vector<double> & coefs){
  for(int i = 0; i < matrix.size(); ++i){
    std::cout << i << std::endl;
    for(int j = 0; j < matrix[i].size(); ++j){
        std::cout << j << std::endl;
        rows.push_back(i + 1);
        columns.push_back(j + 1);
        coefs.push_back((double) matrix[i][j]);
    }
  }
}

void addColumnToGLPKMatrix(std::vector<std::vector<int>> matrix, std::vector<int> & rows, std::vector<int> &columns, std::vector<double> & coefs){
  int currentColumnSize = columns[columns.size() - 1];
  for(int i = 0; i < matrix.size(); ++i){
    rows.push_back(i + 1);
    columns.push_back(currentColumnSize + 1);
    coefs.push_back((double) matrix[i][0]);
  }
}

void displayCppMatrix(std::vector<std::vector<int>> matrix){
  for(int i = 0; i < matrix.size(); ++i){
    for(int j = 0; j < matrix[i].size(); ++j){
      std::cout << matrix[i][j] << " ";
    }
    std::cout << std::endl;
  }
}

void displayGLPKMatrix(std::vector<int> rows, std::vector<int>columns, std::vector<double> coefs){
  for(int i = 0; i < rows.size(); ++i){
    std::cout << rows[i] << ", " << columns[i] << ": " << coefs[i] << std::endl;
  }
}

int main(int argc, char const *argv[]) {
  int startingSize;
  std::vector<int> finalSizes;
  std::vector<int> rowsVector = {0};
  std::vector<int> columnsVector = {0};
  std::vector<double> coefsVector = {0.0};

  int row[1000+1];
  int column[1000+1];
  double coef[1000+1];

  /*std::vector<int> row1 = {1, 2, 3};
  std::vector<int> row2 = {5, 6, 7};
  std::vector<int> row3 = {9, 10, 11};

  std::vector<std::vector<int>> matrix;
  matrix.push_back(row1);
  matrix.push_back(row2);
  matrix.push_back(row3);

  std::vector<int> row4 = {4};
  std::vector<int> row5 = {8};
  std::vector<int> row6 = {12};

  std::vector<std::vector<int>> matrix2;
  matrix2.push_back(row4);
  matrix2.push_back(row5);
  matrix2.push_back(row6);

  displayCppMatrix(matrix);
  transformCppMatrixToGLPKMatrix(matrix, rowsVector, columnsVector, coefsVector);
  addColumnToGLPKMatrix(matrix2, rowsVector, columnsVector, coefsVector);
  displayGLPKMatrix(rowsVector, columnsVector, coefsVector);

  std::copy(rowsVector.begin(), rowsVector.end(), row);
  std::copy(columnsVector.begin(), columnsVector.end(), column);
  std::copy(coefsVector.begin(), coefsVector.end(), coef);*/

  glp_prob *lp;
  double z, x1, x2, x3, x4, y1, y2, y3, y4;

  lp = glp_create_prob();
  glp_set_prob_name(lp, "primal");
  glp_set_obj_dir(lp, GLP_MIN);

  //Constraints
  glp_add_rows(lp, 4);
  glp_set_row_name(lp, 1, "a1");
  glp_set_row_bnds(lp, 1, GLP_FX, 97.0, 97.0);
  glp_set_row_name(lp, 2, "a2");
  glp_set_row_bnds(lp, 2, GLP_FX, 610.0, 610.0);
  glp_set_row_name(lp, 3, "a3");
  glp_set_row_bnds(lp, 3, GLP_FX, 395.0, 395.0);
  glp_set_row_name(lp, 4, "a4");
  glp_set_row_bnds(lp, 4, GLP_FX, 211.0, 211.0);

  glp_add_cols(lp, 4);
  glp_set_col_name(lp, 1, "x1");
  glp_set_col_bnds(lp, 1, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(lp, 1, 1.0);
  glp_set_col_name(lp, 2, "x2");
  glp_set_col_bnds(lp, 2, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(lp, 2, 1.0);
  glp_set_col_name(lp, 3, "x3");
  glp_set_col_bnds(lp, 3, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(lp, 3, 1.0);
  glp_set_col_name(lp, 4, "x4");
  glp_set_col_bnds(lp, 4, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(lp, 4, 1.0);

  std::vector<std::vector<int>> matrix = {{2, 0, 0, 0}, {0, 2, 0, 0}, {0, 0, 3, 0}, {0, 0, 0, 7}};

  transformCppMatrixToGLPKMatrix(matrix, rowsVector, columnsVector, coefsVector);

  std::copy(rowsVector.begin(), rowsVector.end(), row);
  std::copy(columnsVector.begin(), columnsVector.end(), column);
  std::copy(coefsVector.begin(), coefsVector.end(), coef);

  glp_load_matrix(lp, 16, row, column, coef);

  glp_write_lp(lp, NULL, "lp.txt");
  glp_iocp parm;
  glp_init_iocp(&parm);
  parm.presolve = GLP_ON;
  int err = glp_intopt(lp, &parm);
  glp_simplex(lp, NULL);
  /* recover and display results */
  z = glp_get_obj_val(lp);
  x1 = glp_get_col_prim(lp, 1);
  x2 = glp_get_col_prim(lp, 2);
  x3 = glp_get_col_prim(lp, 3);
  x4 = glp_get_col_prim(lp, 4);
  printf("z = %g; x1 = %g; x2 = %g; x3 = %g; x4 = %g\n", z, x1, x2, x3, x4);

  y1 = glp_get_row_dual(lp, 1);
  y2 = glp_get_row_dual(lp, 2);
  y3 = glp_get_row_dual(lp, 3);
  y4 = glp_get_row_dual(lp, 4);
  printf("z = %g; y1 = %g; y2 = %g; y3 = %g; y4 = %g\n", z, y1, y2, y3, y4);
  /* housekeeping */
  
  glp_prob *knapsack;
  double zk;
  int k1, k2, k3, k4;

  knapsack = glp_create_prob();
  glp_set_prob_name(knapsack, "knapsack");
  glp_set_obj_dir(knapsack, GLP_MAX);

  glp_add_rows(knapsack, 1);
  glp_set_row_name(knapsack, 1, "ak1");
  glp_set_row_bnds(knapsack, 1, GLP_UP, 100.0, 100.0);

  glp_add_cols(knapsack, 4);
  glp_set_col_name(knapsack, 1, "a1");
  glp_set_col_bnds(knapsack, 1, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(knapsack, 1, y1);
  glp_set_col_kind(knapsack, 1, GLP_IV);
  glp_set_col_name(knapsack, 2, "a2");
  glp_set_col_bnds(knapsack, 2, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(knapsack, 2, y2);
  glp_set_col_kind(knapsack, 2, GLP_IV);
  glp_set_col_name(knapsack, 3, "a3");
  glp_set_col_bnds(knapsack, 3, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(knapsack, 3, y3);
  glp_set_col_kind(knapsack, 3, GLP_IV);
  glp_set_col_name(knapsack, 4, "a4");
  glp_set_col_bnds(knapsack, 4, GLP_LO, 0.0, 0.0);
  glp_set_obj_coef(knapsack, 4, y4);
  glp_set_col_kind(knapsack, 4, GLP_IV);

  double coefKnapsack[4+1] = {0.0, 45.0, 36.0, 31.0, 14.0};

  for(int i = 0 ; i < 4 ; ++i){
    int ind[2];
    double val[2];

    ind[1] = 1;
    val[1] = coefKnapsack[i + 1];
    glp_set_mat_col(knapsack, i + 1, 1, ind, val);
  }


  glp_write_lp(knapsack, NULL, "knapsack.txt");
  glp_iocp parmk;
  glp_init_iocp(&parmk);
  parmk.presolve = GLP_ON;
  parmk.msg_lev = GLP_MSG_ALL;
  glp_intopt(knapsack, &parmk);

  zk = glp_mip_obj_val(knapsack);
  k1 = glp_mip_col_val(knapsack, 1);
  k2 = glp_mip_col_val(knapsack, 2);
  k3 = glp_mip_col_val(knapsack, 3);
  k4 = glp_mip_col_val(knapsack, 4);
  printf("zk = %g; k1 = %g; k2 = %g; k3 = %g; k4 = %g\n", zk, k1, k2, k3, k4);

  glp_delete_prob(lp);
  glp_free_env();
  return 0;


  return 0;
}
